/*
 tether.c

 Copyright  2002, Larry Barello


 This is really crude code.  Everything is global and unique names.
 It really should be a variable data structure and indirect references.
 GCC and the AVR would actually create faster, denser code and extending
 the code would be simply a matter of making the structure a bit larger.
 */
#define _SFR_ASM_COMPAT 1
#define __SFR_OFFSET 0

#include "hardware.h"
#include "io.h"
#include "eeprom.h"
#include "iomacros.h"
#include "pgmspace.h"
#include "lcdio.h"

#define FLASH __attribute__((progmem))
#define EEPROM __attribute__ ((section(".eeprom")))

static char FLASH sCh0[]    = "0:";
static char FLASH sCh1[]    = " ms 1:";
static char FLASH sCh2[]    = "2:";
static char FLASH sCh3[]    = " ms 3:";
static char FLASH sMs[]     = "ms";
static char FLASH sCalibrate[]="CALIBRATE";

enum
{
    NORMAL,
    CAL_CENTER,
    CAL_RANGE
};

unsigned RCTime(unsigned port, unsigned char bit, unsigned maxtime);
void PulseOut(unsigned time, unsigned port, unsigned char bit);
void ReadCalSwitch(void);
void MeasureInputs(void);
void FilterRange(void);
void StoreCalibration(void);
void DriveOutputs(void);
void UpdateLCD(void);
void ReadCalibration(void);
void ResetRange(void);
void eeprom_ww(unsigned adr, unsigned data);
void Delay(unsigned);
void lcd_putstr(const char *);


#define MAXCHAN 4

struct              // Our dynamic information
{
    int time, current, high, low;
} adc[MAXCHAN];;

struct              // Calibration information (stored in EEPROM)
{
    int center, gain;
} adc_c[MAXCHAN], EEPROM eeAdc_c[MAXCHAN];

char cal, update;                   // Misc stuff


int main(void)
{
    outp(DDRB_INIT, DDRB);
    outp(PORTB_INIT, PORTB);
    outp(DDRD_INIT, DDRD);
    outp(PORTD_INIT, PORTD);
    ReadCalibration();
    lcd_4_init();
    lcd_wr_inst(LCD_4BIT_2LINE);
    lcd_wr_inst(LCD_CLEAR);
    Delay(4);
    lcd_wr_inst(LCD_DISPLAY | LCD_D);   // Display on, Cursor off

    while(1)
    {
        ReadCalSwitch();
        MeasureInputs();
        UpdateLCD();

        switch (cal)
        {
            case NORMAL:
                DriveOutputs();
            break;

            case CAL_CENTER:
                ResetRange();
                cal = CAL_RANGE;
            case CAL_RANGE:
                Delay(10);
                FilterRange();
            break;

            default:
                StoreCalibration();
                cal = NORMAL;
        }
    }
}

void foo(void)
{
    return;
}

void Delay(unsigned ms)
{
    int i;

    while (ms--)
    {
        for (i = 0; i <= 665; i++)
            foo();
    }
}

unsigned LimitRC(unsigned val)
{
    if (val > 1990)
        val = 1990;
    else if (val < 1010)
        val = 1010;
    return val;
}

void DriveOutputs(void)
{
    int i;

    for (i=0; i<MAXCHAN; i++)
    {
        if (adc_c[i].gain != -1)
            adc[i].time = LimitRC(((adc[i].current - adc_c[i].center) * adc_c[i].gain) / 16 + 1500);
        else
            adc[i].time = 1500;
    }
    PulseOut(adc[0].time, RC_PORT, RC_0);
    PulseOut(adc[1].time, RC_PORT, RC_1);
    PulseOut(adc[2].time, RC_PORT, RC_2);
    PulseOut(adc[3].time, RC_PORT, RC_3);
}

void ReadCalSwitch(void)
{
    static char state;
    char current;

    current = RD_CAL();

    if (current != state && state)
    {
        cal++;      // Increment mode on negative transitions.
    }
    state = current;
}
//
// Limit measurement to 500us
//
void MeasureInputs(void)
{
    adc[0].current = RCTime(JOY1_X1, 1000);
    adc[1].current = RCTime(JOY1_Y1, 1000);
    adc[2].current = RCTime(JOY1_X2, 1000);
    adc[3].current = RCTime(JOY1_Y2, 1000);
}

void FilterRange(void)
{
    int i;

    for (i=0; i<MAXCHAN; i++)
    {
        if (adc[i].current > adc[i].high) adc[i].high = adc[i].current;
        if (adc[i].current < adc[i].low) adc[i].low = adc[i].current;
    }
}

void ResetRange()
{
    int i;

    for (i=0; i<MAXCHAN; i++)
    {
        adc[i].low = adc[i].high = adc[i].current;
    }
}

void ReadCalibration(void)
{
    eeprom_read_block((void *)&adc_c, (unsigned)&eeAdc_c, sizeof(eeAdc_c));
}

//
// All calibration numbers are 12.4, so we have fractional
// amounts.
//
// Gain is 12.4 number.  So, rather than multiply by 16
// and divide by 128, I just divide by 8.
//
void StoreCalibration(void)
{
    int i, tmp;
    char *s, *d;

    for (i=0; i<MAXCHAN; i++)
    {
        tmp = adc[i].high - adc[i].low;
        if (tmp > 100)
            adc_c[i].gain = 16000/tmp + 1;
        else
            adc_c[i].gain = -1;
        adc_c[i].center = (adc[i].high + adc[i].low) / 2;
    }
    //
    // Block copy to EEPROM
    //
    s = (char*)&adc_c;
    d = (char*)&eeAdc_c;
    while (s < (char*)&adc_c[MAXCHAN])
        eeprom_wb((unsigned)d++, *s++);
}

void lcd_PutUDec(unsigned val)
{
    char i, lz = 0;

    for (i = 0; val >= 1000; i++)
        val -= 1000;
    lcd_put_char(i+'0');
    lz = 1;
    lcd_put_char('.');

    for (i = 0; val >= 100; i++)
        val -= 100;
    if (i || lz)
    {
    lcd_put_char(i+'0');
        lz = 1;
    }
    for (i = 0; val >= 10; i++)
        val -= 10;
    if (i || lz)
    {
    lcd_put_char(i+'0');
        lz = 1;
    }
//    lcd_put_char(i+'0');
//    lcd_putstr(sMs);
}

void UpdateLCD(void)
{
    if (++update > 50)
    {
        update = 0;
        if (cal == NORMAL)
        {
            lcd_setcursor(LCD_LINE1);
            lcd_putstr(sCh0);
            lcd_PutUDec(adc[0].time);
            lcd_putstr(sCh1);
            lcd_PutUDec(adc[1].time);
            lcd_setcursor(LCD_LINE2);
            lcd_putstr(sCh2);
            lcd_PutUDec(adc[2].time);
            lcd_putstr(sCh3);
            lcd_PutUDec(adc[3].time);
        }
        else
        {
            lcd_wr_inst(LCD_CLEAR);
            Delay(4);
            lcd_putstr(sCalibrate);
        }
    }
}

void lcd_putstr(const char * s)
{
    char c;
    while ((c = PRG_RDB(s++)))
        lcd_put_char(c);
}

