; TetherSup.s
;
; Copyright  2002, Larry Barello
;
; Tuned ASM code to measure & generate pulse widths with 1us resolution
; using an AVR processor @ 8mhz
;

;
; unsigned RCTime(unsigned port, const unsigned char pin, unsigned max)
;
; Measure the RC time constant on a port pin in uS @ 8mhz with a "max" count.
;
	.global	RCTime
	.func	RCTime
RCTime:
	mov	r30, r24
	mov	r31, r25	; port address (e.g. PINB)
	adiw	r30, 0x20	; Shift to data space
	mov	r24, r20
	mov	r25, r21	; Copy over max count
	com	r22
	ldd	r23, Z+1
	and	r23, r22	; Make input
	com	r22
	std	Z+1, r23
1:
	ld	r23, Z
	and	r23, r22	; mask all bits but RC
	brne	2f
	subi	R20, lo8(1)
	sbci	R21, hi8(1)
	brne	1b
2:
	sub	R24, R20
	sbc	R25, R21
	ldd	r23, Z+1
	or	r23, r22	; Make output
	std	Z+1, r23

	ret
	.endfunc
;
; void PulseOut(unsigned width, unsigned char *port, const unsigned char pin)
;
; Generate a posative Pulse on an output pin (e.g. PORTB) configured as output.
; passed # in microseconds @ 8mhz
;
	.global	PulseOut
	.func	PulseOut
PulseOut:
	mov	r30, r22
	mov	r31, r23
	adiw	r30, 0x20	; Shift to data space
	ld	r22, Z
	or	r22, r20
	st	Z, r22
1:
	sbiw	r24, 0
	breq	2f
	sbiw	r24, 1
	nop
	rjmp	1b
2:
	com	r20
	and	r22, r20
	st	Z, r22
	ret
	.endfunc
