/*
lcd.s

Copyright  1998 - 2002 Larry Barello

Author: Larry Barello
        larry@barello.net
*/
#define __SFR_OFFSET 0
#include	"avrx.inc"
#include	"avrx-ctoasm.inc"
#include        "lcd.inc"

;
; Hitachi LCD Control routines for industry standard LCD controllers.
; This code implements a flat (no stack usage) 4bit interface
;
; This code assumes that p1h is trashable
;
#define LCD_PARM p1l    // Register GCC uses to pass/ret a byte/word parameter.
#define LCD_TMP  p1h

        _CODESECTION
LCD_DELAY:      .macro  // number of delay cycles
        rjmp    .       //  4 mhz = 0
        rjmp    .       //  8 mhz = 1
;        rjmp    .       // 12 mhz = 3
        .endm

        _FUNCTION(lcd_fdelay)	// Delay in uSeconds (max 255)
lcd_fdelay:
	LCD_DELAY		// .5us
	dec     LCD_TMP
        nop
        brne	lcd_fdelay
1:
        ret
        _ENDFUNC
;+
;----------------------------------------------
; void lcd_wr_data(uint8_t)
; void lcd_wr_command(uint8_t)
;
; WR LCD Data and Instruction.
;
; PASSED:       LCD_PARM (R25)  = Write Data
; RETURN:
; Registers:    LCD_TMP         = scratch
; Stack:        None
;-
        _FUNCTION(lcd_put_char)
        _PUBLIC(lcd_wr_data)
        _PUBLIC(lcd_putc)
lcd_put_char:
lcd_putc:
lcd_wr_data:
        sbi     LCD_CTL, LCD_RS         ; RS = 1 = DATA
        rjmp    lcd_wr
        _ENDFUNC

        _FUNCTION(lcd_wr_inst)
        _PUBLIC(lcd_wr_inst)
lcd_wr_inst:
        cbi     LCD_CTL, LCD_RS         ; RS = 0 = INSTRUCTION
        _ENDFUNC

        _FUNCTION(lcd_wr)
lcd_wr:
;
; Delay 65 us (ignore busy flag)
;
	ldi	LCD_TMP, 100
	rcall	lcd_fdelay

;
; 4 bit I/O Write operation (LCD_CTL restored)
;
        sbi     LCD_CTL, LCD_E          ; Raise E (do this before capture state
                                        ; of LCD_DATA_OUT in case same port used)
        in      LCD_TMP, LCD_DATA_OUT   ; Save a copy of data port
#if     (LCD_DATA_BITS==LCD_LOW_NIBBLE)
        swap    LCD_PARM                ; Put out High nibble first
#endif
        cbr     LCD_TMP, LCD_DATA_BITS
        eor     LCD_TMP, LCD_PARM       ; Or in the data (high and low nibbles)
        cbr     LCD_PARM, LCD_DATA_BITS ; cancel data in PARM
        eor     LCD_TMP, LCD_PARM       ; Cancel out other nibble
        out     LCD_DATA_OUT, LCD_TMP

        LCD_DELAY                       ; Need to delay 500 ns (total)
        cbi     LCD_CTL, LCD_E          ; lower E

        LCD_DELAY                       ; Need to delay 500 ns

        swap    LCD_PARM                ; Put out the other nibble
        cbr     LCD_TMP, LCD_DATA_BITS
        or      LCD_TMP, LCD_PARM       ; Low nibble

        sbi     LCD_CTL, LCD_E
        out     LCD_DATA_OUT, LCD_TMP
        LCD_DELAY                       ; Need to delay 500 ns (total)
        cbi     LCD_CTL, LCD_E

        ret
        _ENDFUNC
;
;--------------------------------------------------
;
; lcd_4_init
;
; Switches reset LCD to 4 bit I/F mode.
;
; PASSED:       Nothing
; RETURNS:
; USES:         LCD_PARM (R24), LCD_TMP, T
; NOTES:        Initial read of Busy Flag and write to controller is 8 bit,
;               susequent i/o is 4 bit (dual r/w cycles).  Need to do a second
;               function set to get all bits to the lcd display.
;
        _FUNCTION(lcd_4_init)

lcd_4_init:
        ldi     LCD_PARM, LCD_CTL_DDR_INIT
        out     LCD_CTL_DDR, LCD_PARM

        in      LCD_TMP, LCD_DATA_DDR
        sbr     LCD_TMP, LCD_DATA_BITS
        out     LCD_DATA_DDR, LCD_TMP

        cbi     LCD_CTL, LCD_RS         ; RS = 0 = INSTRUCTION
        cbi     LCD_CTL, LCD_E          ; Switch to output mode

	ldi	LCD_PARM, 160		; Delay 16 ms
1:
	ldi	LCD_TMP, 100
	rcall	lcd_fdelay
	dec	LCD_PARM
	brne	1b
#if 0
        in      LCD_TMP, LCD_DATA_IN
        cbr     LCD_TMP, LCD_DATA_BITS
#if     (LCD_DATA_BITS==0x0F)
        ori     LCD_TMP, (LCD_8BIT_1LINE)>>4
#else
        ori     LCD_TMP, LCD_8BIT_1LINE
#endif
        out     LCD_DATA_OUT, LCD_TMP

	rcall	ToggleE

	ldi	LCD_PARM, 40		; Delay 4 ms
1:
	ldi	LCD_TMP, 100
	rcall	lcd_fdelay
	dec	LCD_PARM
	brne	1b

	Rcall	ToggleE

;	ldi	LCD_PARM, 40		; Delay 4 ms
1:
	ldi	LCD_TMP, 100
	rcall	lcd_fdelay
;	dec	LCD_PARM
;	brne	1b
#endif
        in      LCD_TMP, LCD_DATA_IN
        cbr     LCD_TMP, LCD_DATA_BITS
#if     (LCD_DATA_BITS==0x0F)
        ori     LCD_TMP, (LCD_4BIT_2LINE)>>4
#else
        ori     LCD_TMP, LCD_4BIT_2LINE
#endif
        out     LCD_DATA_OUT, LCD_TMP

        Rcall	ToggleE
	ldi	LCD_PARM, 40		; Delay 4 ms for mode change
1:
	ldi	LCD_TMP, 100
	rcall	lcd_fdelay
	dec	LCD_PARM
	brne	1b

	ret
        _ENDFUNC

        _FUNCTION(ToggleE)
ToggleE:
	sbi	LCD_CTL, LCD_E
	LCD_DELAY
	cbi	LCD_CTL, LCD_E
	ret
	_ENDFUNC
        _END

